# BSTree.s
# 
# Binary Search TREE 
# 
# Node: 3 words: 
#       Left (pointer), Data (character), Right (pointer)
# 
# Header (pointer) initialized to 0; points to root
#  
	.data                   # Data declaration section
title:  .asciiz   "Binary Search Tree Demonstration"
head:   .word     0             # (could have been a register)
prompt: .asciiz   "\nBSTree OP: F, I, D, P, or E: "
error:  .asciiz   "\nUndefined Operation; try again: "
value:  .asciiz   "\nWhat (character) value? "
nofmsg: .asciiz   "\nThat value is not in the BSTree "
fmsg:   .asciiz   "\nThe value was found "
noI:    .asciiz   "\nThat value is already in the BSTree "
Emsg:   .asciiz   "\nThe tree is empty "
Pmsg:   .asciiz   "\nBSTree: "
bye:    .asciiz   "\nGoodbye! "

# 
	  .text
main:                           # Start of code section
         li       $v0,4
         la       $a0,title
         syscall         
loop:    li       $v0,4
         la       $a0,prompt 
         syscall                # Print prompt
         li       $v0,12        # No MIPSter	
         syscall                # Get reply
         andi     $v0,$v0,0xFF  # Mask character
# Condensed tests for legal character (save source space)
 beq $v0,'F',F; beq $v0,'f',F; beq $v0,'I',I; beq $v0,'i',I
 beq $v0,'D',D; beq $v0,'d',D; beq $v0,'P',P; beq $v0,'p',P
 beq $v0,'E',E; beq $v0,'e',E
# Not a legal request
         li       $v0,4
         la       $a0,error
         syscall
         j        loop          # main interface loop

# FIND 
F:       
         li       $v0,4
         la       $a0,value
         syscall                # prompt for (char) value
         li       $v0,12        # no MIPSter
         syscall                # puts (char) value in $v0
         move     $a1,$v0
         la       $a0,head
         jal      FF            # call on FF function
         bltz     $v0,Fempty    # empty tree
         lw       $v0,4($v0)
         bne      $v0,$a1,notin # wrong character
# found
         li       $v0,4         # for debug purposes
         la       $a0,fmsg      # usually just return
         syscall                # pointers
         j        loop
# not found                        
notin:   li       $v0,4         # for debug purposes
         la       $a0,nofmsg    # usually just return
         syscall                # flag like -1
         j        loop
# empty tree
Fempty:  li       $v0,4         # for debug purposes
         la       $a0,Emsg      # usually just return
         syscall                # a special flag
         j        loop

# 
# Find Function (not recursive)
# $a0 -> Header; $a1 = key (character)
# returns address of Node containing character in $v0
# or whose child it should be or -1 if empty tree
# In $v1 will be put the parent of $v0 if it exists 
# (for "delete" purposes only)
#
FF:                             # <- entry point
         sub      $sp,$sp,4     # save register
         sw       $t0,0($sp)    # just $t0 changed
         li       $v0,-1        # initialize return
         li       $v1,-1        # initialize return
         lw       $t0,0($a0)    # what's in header
         beqz     $t0,FFreturn  # empty tree
         move     $v0,$t0       # $v0 -> root ($v1=-1)
FFloop:  lw       $t0,4($v0)    # get character
         blt      $a1,$t0,FFleft
         bgt      $a1,$t0,FFright
         b        FFreturn      # equal, so found
FFright: lw       $t0,8($v0)    # get right ptr
         beqz     $t0,FFreturn
         move     $v1,$v0       # fix parent
         move     $v0,$t0       # go down
         b        FFloop
FFleft:  lw       $t0,0($v0)    # get left ptr
         beqz     $t0,FFreturn
         move     $v1,$v0       # fix parent
         move     $v0,$t0       # go down
         b        FFloop
FFreturn:sw       $t0,0($sp)    # restore register
         add      $sp,$sp,4     # just $t0
         jr       $ra

# INSERT
I:
         li       $v0,4
         la       $a0,value
         syscall                # prompt for value
         li       $v0,12        # no MIPSter
         syscall                # puts value in $v0
         move     $a1,$v0       # $a1 has value to insert
         la       $a0,head
         jal      FF            # use find (FF) function
         bltz     $v0,Iempty    # was empty tree
#
         lw       $t0,4($v0)    # get character at $v0
         blt      $a1,$t0,IL    # into left child
         bgt      $a1,$t0,IR    # into right child
# equal, so already there
         li       $v0,4         # print already
         la       $a0,noI       # there message
         syscall
         j        loop
# insert into right child
IR:      move     $t1,$v0       # save old node pointer
         li       $v0,9         # getmem
         li       $a0,12        # 3 words (initially 0)
         syscall                # $v0 -> new node
         sw       $a1,4($v0)    # put in proper place
         sw       $v0,8($t1)    # right pointer fixed
         j        loop
# insert into left child
IL:      move     $t1,$v0       # save old node pointer
         li       $v0,9         # getmem
         li       $a0,12        # 3 words (initially 0)
         syscall                # $v0 -> new node
         sw       $a1,4($v0)    # put in proper place
         sw       $v0,0($t1)    # left pointer fixed      
         j        loop
# empty tree
Iempty:  li       $v0,9         # getmem
         li       $a0,12        # 3 words (initially 0)
         syscall                # $v0 -> new node
         sw       $v0,head      # head -> root
         sw       $a1,4($v0)    # put char there
         j        loop

# DELETE
# NOT implemented yet (exercise project)
         .data
notimp:  .asciiz  "\nDelete function not implemented\n"
         .text
D:
         li       $v0,4
         la       $a0,notimp
         syscall
         j        loop

# PRINT
P:
         la       $a0,head
         lw       $t0,0($a0)    # what's in header
         beqz     $t0,Pempty    # special empty tree case
         la       $a0,Pmsg      # print heading
         li       $v0,4
         syscall
         move     $a0,$t0       # $a0 -> root
         jal      PFrec         # call the recursive fnctn
         j        loop
Pempty:  li       $v0,4         # make this a special case
         la       $a0,Emsg      # so recursive function can
         syscall                # always expect the root node
         j        loop
#
# Print Function (recursive)
# $a0 -> root node [not header since not a node]
# Recursive implementation
# Inorder traversal (LNR)
#
PFrec:                          # <- entry point
         sub      $sp,$sp,16    #\ 
         sw       $a0,0($sp)    # \
         sw       $v0,4($sp)    #  > save registers
         sw       $ra,8($sp)    # /
         sw       $t0,12($sp)   #/
# Basecase
         beqz     $a0,PFret     # root pointer zero
# Left
         move     $t0,$a0
         lw       $a0,0($a0)
         jal      PFrec         # recursive call
         move     $a0,$t0
# Node
         move     $t0,$a0
         lw       $a0,4($a0)    # print the character
         li       $v0,11        # at the node
         syscall
         move     $a0,$t0
# Right
         move     $t0,$a0
         lw       $a0,8($a0)
         jal      PFrec         # recursive call
         move     $a0,$t0
# Return
PFret:   lw       $t0,12($sp)   #\ 
         lw       $ra,8($sp)    # \
         lw       $v0,4($sp)    #  > restore registers
         lw       $a0,0($sp)    # /
         add      $sp,$sp,16    #/
         jr       $ra

# EXIT
E:       li       $v0,4
         la       $a0,bye
         syscall      
         li       $v0, 10       # exits program
         syscall



	
# END BSTree.s
