## PGAXDemo.s                      
# 
#   Driver to test PGAexceptions.s as an extended exception handler.
#   Modification of code originally written by Dr. Herman Harrison.
#   It uses new operations involving the timer and buffered I/O.
# 
         .data
title:   .asciiz  "\nDemonstrate Coprocessor _0 Exceptions:\n" 
options: .ascii   "\n\nTest of I/O and Timer Interrupts:"
         .ascii   "\n---------------------------------"
         .ascii   "\n1)Type character(s) then <Enter> key"
         .ascii   "\n2)Type \"<Ctrl-T>\" Toggle Timer messages on/off"
         .ascii              " (default=off)"
         .ascii   "\n3)Type \"<Ctrl-V>\" rapidly to View timer Values"
         .ascii              " (default=100)"
         .ascii   "\n4)Type \"<Ctrl-S>\" Make the timer speed slower"        
         .ascii   "\n5)Type \"<Ctrl-F>\" Make the timer speed faster"        
         .ascii   "\n6)Type \"<Ctrl-X>\" eXit from the program"
         .ascii   "\n7)Type \"<Ctrl-O>\" to see these Options again"
         .asciiz  "\n\n"
CountTick: .asciiz "\n[Count:Tick]="
newvalue:  .asciiz "\nNew Tick value=0:"
colon:     .asciiz ":"
confirm:   .asciiz "\nAre you sure(y/n)?"
bye:       .asciiz "\nGood bye\n"
tm_display:  .word    0           # initialize to OFF

        .text
        .globl  main
main:
        la      $a0, title
        li      $v0, 4
        syscall
restart:
        la      $a0, options
        li      $v0, 4
        syscall
        lw      $a0, tm_display  # Turn on timer display messages
        jal     .TIMER_display
mainloop:
        jal     .IO_getc       # returns -1 if no char /\ 
        blez    $v0, mainloop  # wait for some input   \/  
        move    $s0, $v0
        beq     $s0, 10, bufferit # <Enter> (actually LF)
        beq     $s0, 13, bufferit # <Enter> (actually CR)
        bge     $s0, 32, bufferit # printable character
        b       what           # not printable character
bufferit: 
        move    $a0, $v0       # put character in buffer
        jal     .IO_putc
# Analyse command
what:   bne     $s0, 20, notT  # <Ctrl-T> Toggle timer display on/off
        lw      $a0, tm_display
        xor     $a0, $a0, 1
        sw      $a0, tm_display
        jal     .TIMER_display    
notT:
        bne     $s0, 19, notS  # <Ctrl-S> make slower 
        la      $a0, newvalue  # by doubling timer value
        li      $v0, 4
        syscall
        jal     .TIMER_get
        sll     $a0, $v0, 1    # *2
        jal     .TIMER_set
        li      $v0, 1
        syscall
notS:
        bne     $s0, 6, notF   # <Ctrl-F> make faster
        la      $a0, newvalue  # by halfing timer value
        li      $v0, 4
        syscall
        jal     .TIMER_get
        srl     $a0, $v0, 1    # /2
        jal     .TIMER_set
        li      $v0, 1
        syscall
notF:
        bne     $s0, 22, notV  # <Ctrl-V> View timer Value
        la      $a0, CountTick
        li      $v0, 4
        syscall
        jal     .TIMER_getClock  # get current Clock value
        move    $a0, $v0
        li      $v0, 1
        syscall
        la      $a0, colon     # print ":"
        li      $v0, 4
        syscall
        jal     .TIMER_get       # get current Timer tick value
        move    $a0, $v1
        li      $v0, 1
        syscall
notV:
        bne     $s0, 15, notO  # <Ctrl-O> show Options again
        la      $a0, options
        li      $v0, 4
        syscall
notO:
        beq     $s0, 24, eXit  # <Ctrl-X> eXit from program
        bne     $s0, 0x0a, mainloop # <Enter> (actually ^J = LF)
        jal     .IO_flush      # flush kernel I/O output buffer
        b       mainloop
#-------Leave main loop---------------------------------------
eXit:
        li      $a0, 0         # Turn off Timer display messages
        jal     .TIMER_display  
        jal     .IO_flush      # clear out I/O buffer
floop:  jal     .IO_empty      # wait for flush to complete
        bnez    $v0, floop     
        la      $a0, confirm   # Confirmation request
        li      $v0, 4
        syscall
rloop:  jal     .IO_getc       # check for a "y" response
        blez    $v0, rloop
        bne     $v0, 'y', restart # restart and print out options
#-------Exit to kernel----------------------------------------         
        la      $a0, bye
        li      $v0, 4
        syscall     
        li      $v0, 10
        syscall
