
### PGAexceptions.s ### A modified version of "exceptions.s"    #
### as distributed with PCSpim Version 7.2 by James Larus.      #
### Developed by Dr. Hermann Harrison of The University of      # 
### Texas at Dallas. It may be used for any program but was     #
### designed to demonstrate features in "PGAXDemo.s".           #
###                                                             # 
### It defines new user I/O operations named ".IO_getc" and     #
### ".IO_putc" which get or put characters through a buffer     #
### that may be checked to be empty with ".IO_empty", and       #
### emptied with ".IO_flush". These are accessed with jal.      #
###                                                             #
### In addition, a timer is introduced with the new operations  #
### ".TIMER_set", ".TIMER_get", ".TIMER_display", and           #
### ".TIMER_getClock". These are also accessed with jal.        #
#===============================================================#
# SPIM S20 MIPS simulator.
# The default exception handler for spim.
#
# Copyright (C) 1990-2004 James Larus, larus@cs.wisc.edu.
# ALL RIGHTS RESERVED.
#
# SPIM is distributed under the following conditions:
#
# You may make copies of SPIM for your own use and modify 
# those copies.
#
# All copies of SPIM must retain my name and copyright notice.
#
# You may not sell SPIM or distributed SPIM in conjunction 
# with a commercial product or service without the expressed 
# written consent of James Larus.
#
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS 
# OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE 
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
# PARTICULAR PURPOSE.
#
# $Header: $
# Define the exception handling code.  This must go first!
#=============================================================
#
#===KERNEL DATA DEFINITIONS===================================
        .kdata
	
#--Kernel Data Variables---------------	
__m01_:  .asciiz  "  Exception "
__m02_:  .asciiz  " occurred and ignored\n"
__e00_:  .asciiz  "  [Interrupt] "
__e01_:  .asciiz	"  [TLB]"
__e02_:  .asciiz	"  [TLB]"
__e03_:  .asciiz	"  [TLB]"
__e04_:  .asciiz	"  [Address error in inst/data fetch] "
__e05_:  .asciiz	"  [Address error in store] "
__e06_:  .asciiz	"  [Bad instruction address] "
__e07_:  .asciiz	"  [Bad data address] "
__e08_:  .asciiz	"  [Error in syscall] "
__e09_:  .asciiz	"  [Breakpoint] "
__e10_:  .asciiz	"  [Reserved instruction] "
__e11_:  .asciiz	"  [Coprocessor error]"
__e12_:  .asciiz	"  [Arithmetic overflow] "
__e13_:  .asciiz	"  [Trap] "
__e14_:  .asciiz	""
__e15_:  .asciiz	"  [Floating point] "
__e16_:  .asciiz	""
__e17_:  .asciiz	""
__e18_:  .asciiz	"  [Coproc 2]"
__e19_:  .asciiz	""
__e20_:  .asciiz	""
__e21_:  .asciiz	""
__e22_:  .asciiz	"  [MDMX]"
__e23_:  .asciiz	"  [Watch]"
__e24_:  .asciiz	"  [Machine check]"
__e25_:  .asciiz	""
__e26_:  .asciiz	""
__e27_:  .asciiz	""
__e28_:  .asciiz	""
__e29_:  .asciiz	""
__e30_:  .asciiz	"  [Cache]"
__e31_:  .asciiz	""

__excp_: .word __e00_, __e01_, __e02_, __e03_, __e04_, __e05_
         .word __e06_, __e07_, __e08_, __e09_, __e10_, __e11_
         .word __e12_, __e13_, __e14_, __e15_, __e16_, __e17_
         .word __e18_, __e19_, __e20_, __e21_, __e22_, __e23_
         .word __e24_, __e25_, __e26_, __e27_, __e28_, __e29_
         .word __e30_, __e31_

__s01_:   .word 0
__s02_:   .word 0

#--Exceptions in Branch Delay Slot---------------
_bds_msg: .asciiz  "\nException in Branch_Delay_Slot"
#--

#--TIMER Data Variables---------------
_TIMER_MSG:     .asciiz  "\nTIMER_events="
_TIMER_CLOCK:   .word 0
_TIMER_DISPLAY: .word 0

#--I/O Buffers/Data Variables---------            
_bufsiz:     .word  128
_bufin:      .space 128
_bufin_end:                   # Must immediately follow "bufin:"
_bufin_put:  .word  _bufin    # input buffer "producer" pointer
_bufin_get:  .word  _bufin    # input buffer "consumer" pointer
_bufout:     .space 128
_bufout_end:                  # Must immediately follow "bufout:"
_bufout_put: .word  _bufout   # output buffer "producer" pointer
_bufout_get: .word  _bufout   # output buffer "consumer" pointer

#====KERNEL INTERRUPT SERVICE ROUTINES==========================
# This is the exception handler code that the processor runs when
# an exception occurs. It only prints some information about the
# exception, but can server as a model of how to write a handler.
#
# Because we are running in the kernel, we can use $k0/$k1 
# without saving their old values.
#---------------------------------------------------------------
# This is the exception vector address for MIPS32:
	.ktext 0x80000180
	
#(1)====SAVE THE CPU ENVIRONMENT/STATE===========================

      .set   noat           #                these avoid 
      move   $k1 $at	    # Save $AT($1)
      .set   at             #                warning messages
      sw     $v0, __s01_    # Not re-entrant, can't trust $sp
      sw     $a0, __s02_    # But, we need to use these registers
#***************************************************************
 	mfc0   $k0, $13       # C0_CAUSE register($13)
      nop                   ##Delay Slot
	srl    $a0, $k0, 2    # Extract Exc_Code Field
# If External Interrupt, do not print information
	andi   $a0, $a0, 0x1f
      beqz   $a0,_Ext_Interrupts  ####Ext Interrupt Handlers#####
      nop                   # Branch-delay-slot
    
#(2)===INTERNAL EXCEPTION HANDLERS===============================
# Print information about exception. (Could handle them here)
      li     $v0, 4		# syscall 4 (print_str)
	la     $a0, __m01_
	syscall

	li     $v0, 1		# syscall 1 (print_int)
	srl    $a0, $k0 2	      # Extract ExcCode Field
	andi   $a0, $a0 0x1f
	syscall

	li     $v0, 4		# syscall 4 (print_str)
	andi   $a0, $k0 0x7c
	lw     $a0, __excp_($a0)
	nop
	syscall
	
      andi   $a0, $k0, 0x7c
	bne    $a0, 0x18, _ok_pc # Bad PC exception requires 
	nop                     # special case

	mfc0   $a0, $14	    	# EPC
      nop                     ##Delay Slot 
	andi   $a0, $a0, 0x3	# Is EPC word-aligned?
	beq    $a0, 0, _ok_pc
	nop
	li     $v0, 10     	# Exit on really bad PC
	syscall

_ok_pc:
	li     $v0, 4   		# syscall 4 (print_str)
	la     $a0, __m02_
	syscall
	
	andi   $a0, $k0, 0x7c   # Extract Exc_Code Field
	bne    $a0, $zero, _ret	# 0 means exception was interrupt
	nop

#(3)===EXTERNAL INTERRUPT HANDLERS==============================
# External/Interrupt-specific code goes here!
# Don't skip instruction at EPC since it has not executed.

_Ext_Interrupts:

#---------TIMER--------------------------------------------
_timer:
    andi   $a0, $k0, 0x8000 # Bit 15 means timer interrupt
    beqz   $a0, _timer_end
    nop                     # Branch-delay-slot 
#   ----------------------
    mtc0   $zero, $9        # Reset CO_Timer_Register
    lw     $a0, _TIMER_CLOCK # Increment the clock tick count
    lw     $v0, _TIMER_DISPLAY # Load delay slot
    addi   $a0, $a0, 1     
    sw     $a0, _TIMER_CLOCK
#   -----------------------
    beqz   $v0, _timer_end  # Check for display toggle
    nop                     # Branch-delay-slot 
    la     $a0, _TIMER_MSG
    li     $v0, 4 
    syscall
    lw     $a0, _TIMER_CLOCK
    li     $v0, 1
    syscall
_timer_end:

#---------TERMINAL INPUT --------------------------------------
_IO_in:
    andi   $a0, $k0, 0x0800 # Bit 11 indicates keyboard input
    beqz   $a0, _IO_out
    nop                     # Branch-delay-slot
#   ------------------------
    li     $a0, 0xffff0000  # Read the character from the input device
    lw     $v0, 4($a0)
    lw     $a0, _bufin_put
    nop                     # Load-delay-slot
    sb     $v0, 0($a0)      # Store character in buffer
    
    addiu  $a0, $a0, 1      # Increment buffer input pointer
    la     $v0, _bufin_end  # Check for buffer wrap
    bne    $v0, $a0, _IO_in1
    nop                     # Branch-delay-slot
    la     $a0, _bufin      # (BDS)Wrap to start of buffer
_IO_in1:
    sw     $a0, _bufin_put
    sb     $zero, 0($a0)    ##### needs to handle buffer overrun
#---------TERMINAL OUTPUT---------------------------------------
_IO_out:
    andi   $a0, $k0, 0x400  # Bit 10 indicates display output
    beqz   $a0, _IO_out_1
    nop                     # Branch-delay-slot
#   ---------------------
    lw     $a0, _bufout_get # Check for character to output
    nop                     # Load-delay-slot
    lbu    $v0, 0($a0)
    nop                     # Load-delay-slot
    beqz   $v0, _IO_out_1   # buffer empty?
    nop                     # Branch-delay-slot
    sb     $zero, 0($a0)    # Clear current buffer location
    addiu  $a0, $a0, 1      # Increment buffer output pointer
    sw     $a0, _bufout_get # Update the pointer
    li     $a0, 0xffff0000  # Write the character to output device
    sw     $v0, 12($a0)
    
    lw     $a0, _bufout_get # Check for buffer wrap
    la     $v0, _bufout_end
    bne    $v0, $a0, _IO_out_1
    nop                     # Branch-delay-slot 
    la     $a0, _bufout     # Wrap to start of buffer
    sw     $a0, _bufout_get # Update the pointer
_IO_out_1:
#-------------------------------
    b      _ret_1
    nop                     # Branch-delay-slot
    
#(4)====RESTORE THE CPU ENVIRONMENT/STATE=================
_ret:                 
# Return from (non-interrupt) exception. Skip offending 
# instruction at EPC to avoid infinite loop.
      li     $a0, 0x80000000
      and    $a0, $a0, $k0  # check the BD bit
      beqz   $a0, _ret_2    
      nop                   # Branch-delay-slot
#///////////////////////exception in Branch_Delay_Slot
      la     $a0, _bds_msg
      li     $v0, 4 
      syscall
      b      _ret_1     #### ASSUMES ONLY "NOP" in BDS
      nop                   # Branch-delay-slot
_ret_2:
	mfc0   $k0, $14       # Bump EPC register
      nop                   ##Delay Slot 
	addiu  $k0, $k0, 4    # Skip faulting instruction
      mtc0   $k0, $14
 
# Restore registers and reset processor state
_ret_1:
      mtc0   $zero, $13     # Clear Cause register 
#*************************************************	
# Return from exception on MIPS32:
	lw     $a0, __s02_	    # Restore other registers
	lw     $v0, __s01_
	.set   noat
	move   $at, $k1	    # Restore $at
	.set   at 
# *** NO PSEUDO INSTRUCTION PASSED THIS POINT IN HANDLER ***	
	eret          # New instruction in MIPS32; replaces rfe
      nop
#=====================================================================

#(A)==========KERNEL INITIALIZATION w/EXCEPTION PROCESSING============
_init_kernel:
    addiu  $sp, $sp, -4     # Push
    sw     $ra, 0($sp)      # Save return address
    mtc0   $zero, $12       # DISABLE ALL interrupts(C0_STATUS=$12)
    mtc0   $zero, $13       # Clear/ACK ALL interrupts(C0_CAUSE=$13)
#---------Reset IO Interface--------------------------------------
    la     $a0, 0xFFFF0000  # RESET Console I/O to polled(default)
    sw     $zero, 0($a0)    # Disable receiver(0xffff0000)
    sw     $zero, 8($a0)    # Disable transmitter(0xffff0008)
#---
    jal    _IO_init         # Initialization for IO interrupts
    nop                     # Branch-delay-slot
#----------Reset/Enable TIMER Interface----------------------------
    mtc0   $zero, $9        # CLEAR CO_Timer_Register($9)
    mtc0   $zero, $11       # Clear CO_Compare_Register($11)
#---                             (Disable compare)
    li     $a0,   100       # Compare timer value (default=100)
    li     $a1,   0         # Turn off debug mode (default=off)
    jal    _timer_init      # Setup Timer for interrupts
    nop                     # Branch-delay-slot
#----------Initialize COPROCESOR_0 Interface (Status/Cause)
    mtc0   $zero, $13       # CLEAR entire C0_CAUSE register
    mfc0   $a0, $12         ### Configure C0_STATUS Register
    nop                     ##Delay Slot 
    ori    $a0, $a0, 0x0011 ### Select User mode and enable 
#                               global interrupts
    mtc0,  $a0, $12         # ENABLE selected interrupts in 
#                               C0_STATUS(0x8c11)
#---------------------------------------------------------------
    lw     $ra, 0($sp)      # Restore return address
    addiu  $sp, $sp, 4      # Pop 
    jr     $ra
    nop                     # Branch-delay-slot
    
#(B)---------TIMER INITIALIZATION-------------------------------
_timer_init:             # $a0 = compare reg, $a1 = debug on/off
   mtc0   $a0,   $11     # SET CO_Compare_Register($11)
   sw     $a1,   _TIMER_DISPLAY # set/reset debug mode
   sw     $zero, _TIMER_CLOCK   # clear Timer clock
   #---------- Select TIMER Interrupt
   mfc0   $a0, $12         ### Configure C0_STATUS Register
   nop                     ##Delay Slot 
   ori    $a0, $a0, 0x8000 ### Select/enable TIMER
   mtc0,  $a0, $12         ### Set interrupt mask in C0_STATUS
   jr     $ra
   nop                   # Branch-delay-slot   
#-----------  
#(B)=========USER ACCESSABLE TIMER ROUTINES=====================
_setTimer:               # $a0 = compare register tick count
    mtc0   $zero, $9     # CLEAR CO_Timer_Register($9)
    mtc0   $a0,   $11    # SET CO_Compare_Register($11)
    sw     $zero, _TIMER_CLOCK # clear Timer clock
    jr     $ra
    nop                  # Branch-delay-slot   
#-----------    
_getTimer:               # $v0 returns current timer value
    mfc0   $v1, $9       # READ CO_Timer_Register($9)
    mfc0   $v0, $11      # READ CO_Compare_Register($11)
    jr     $ra
    nop                  # Branch-delay-slot    
    #-----------    
_getTmClock:             # $v0 returns current timer cycle count
    lw     $v0, _TIMER_CLOCK
    jr     $ra
    nop                  # Branch-delay-slot    
#-----------    
_displayTimer:           # display print $a0=(on=1, off=0)
    sw     $a0, _TIMER_DISPLAY
    jr     $ra
    nop                  # Branch-delay-slot
#(C)---------TERMINAL INITIALIZATION-----------------------------    
_IO_init:
#---------Enable I/O INTR here
    la     $v0, 0xFFFF0000    # Setup Console I/O for interrupts
    li     $v1, 2             # Load interrupt enable bits
    sw     $v1, 0($v0)        # SET receiver on
    sw     $v1, 8($v0)        # SET transmitter on
#----------Initialize I/O BUFFERS
    la     $a0, _bufin
    sw     $a0, _bufin_put
    sw     $a0, _bufin_get
    sw     $zero, 0($a0)
    la     $a0, _bufout
    sw     $a0, _bufout_put
    sw     $a0, _bufout_get
    sw     $zero, 0($a0)
#---------- Select I/O Interrupts
    mfc0   $a0, $12           ### Configure C0_STATUS Register
    nop                       ### Delay Slot 
    ori    $a0, $a0, 0x0c00   ### Select/enable IO
    mtc0,  $a0, $12           ### Set interrupt mask in C0_STATUS
    jr     $ra
    nop       # Branch-delay-slot    
#=========USER ACCESSABLE IO I/O ROUTINES========================
#---------TERMINAL INPUT--------------------------------------
_IO_getc:
    lw     $a0, _bufin_get    # Read character from kernel buffer
    nop                       # Load-delay-slot
    lbu    $v0, 0($a0)
    nop                       # Load-delay-slot
    beqz   $v0, _tgetc_2      # Check for empty buffer
    nop                       # Branch-delay-slot
    sb     $zero, 0($a0)      # Clear current buffer location
    addiu  $a0, $a0, 1        # Increment buffer output pointer
    la     $v1, _bufin_end    # Check for buffer wrap
    bne    $v1, $a0, _tgetc_1
    nop                       # Branch-delay-slot
    la     $a0, _bufin        # Wrap to start of buffer
_tgetc_1:
    sw     $a0, _bufin_get
    jr     $ra
    nop                       # Branch-delay-slot   
#-------------------------------
_tgetc_2:
    li     $v0, -1
    jr     $ra
    nop       # Branch-delay-slot   
#---------TERMINAL OUTPUT---------------------------------------
_IO_putc:
    lw     $v0, _bufout_put   # Check for empty buffer
    nop                       # Load-delay-slot
    lb     $v1, 0($v0)
    nop                       # Load-delay-slot
    bnez   $v1, _tputc_2
    nop                       # Branch-delay-slot 
    lw     $v0, _bufout_put   # Check for full buffer
    nop                       # Load-delay-slot
    lb     $v1, 0($v0)
    nop                       # Load-delay-slot
    bnez   $v1, _tputc_2      # buffer ?
    nop                       # Branch-delay-slot
    sb     $a0, 0($v0)        # Write character to kernel buffer
    
    addiu  $a0, $v0, 1        # Increment buffer output pointer
    la     $v0, _bufout_end   # Check for buffer wrap
    bne    $v0, $a0, _tputc_1
    nop                       # Branch-delay-slot
    la     $a0, _bufout       # wrap to start of buffer
_tputc_1:
    sw     $a0, _bufout_put   # Update the pointer
    li     $v0, 1             # successful
    jr     $ra
    nop                       # Branch-delay-slot   
#-------------------------------
_tputc_2:
    li     $v0, 0             # failed
    jr     $ra
    nop                       # Branch-delay-slot   
#---------TERMINAL FLUSH---------------------------------------
_IO_flush:
    lw     $a0, _bufout_get   # Check for character to output
    nop                       # Load-delay-slot
    lbu    $v0, 0($a0)
    nop                       # Load-delay-slot
    beqz   $v0, _tflush_3     # buffer empty?
    nop                       # Branch-delay-slot
#-----------    
    sb     $zero, 0($a0)      # Clear current buffer location
    addiu  $a0, $a0, 1        # Increment buffer output pointer
    la     $v1, _bufout_end
    bne    $v1, $a0, _tflush_1
    nop                       # Branch-delay-slot
    la     $a0, _bufout       # Wrap to start of buffer
_tflush_1: 
    sw     $a0, _bufout_get   # Update the pointer
_tflush_2:    
    li     $a0, 0xffff0000    # Write the character to output device
    lw     $v1, 8($a0)        # Poll to output first character
    nop                       # Load-delay-slot
    andi   $v1, $v1, 1
    beqz   $v1, _tflush_2
    nop                       # Branch-delay-slot
    sw     $v0, 12($a0)       # Write character to output device
_tflush_3:  
    jr     $ra
    nop                       # Branch-delay-slot
#---------TERMINAL EMPTY?---------------------------------------
_IO_empty:
    lw     $a0, _bufout_get   # Check for character to output
    nop                       # Load-delay-slot
    lbu    $v0, 0($a0)
    jr     $ra
    nop                       # Branch-delay-slot
       
#----------------------------------------------------------------
#=======END OF KERNEL RESIDENT CODE==============================

#=======KERNEL INITIALIZATION/USER PROGRAM LINKAGE======================
	.text
	.globl __start
__start:
# Standard startup code.  Invoke the routine "main" with arguments:
#	main(argc, argv, envp)
	lw     $a0, 0($sp)	# argc
	addiu  $a1, $sp, 4	# argv
	addiu  $a2, $a1, 4	# envp
	sll    $v0, $a0, 2
	addu   $a2, $a2, $v0

# ---Initialize kernel/exception handlers--------------------------
# This is far away in the kernel (different 256K block) so a jump
# register is needed. Other handlers could be initialized here.
      la     $ra, _init_kernel
      jalr   $ra
      nop                     # Branch-delay-slot
# ---End of kernel/exception handlers initialization -------------- 
  
# Finally jump to the user's program starting at "main":
	jal    main
	nop

	li     $v0, 10          # syscall 10 (exit)
	syscall

#======================================================================		
# Special linkage: kernel and user code reside in very far apart 256MB 
# blocks so special code is necessary. The following allows user access
# to the IO and TIMER interface routines written above.
#-----------------Memory Mapped I/O Stuff------------------------------
        .globl  .IO_getc     #  ;$v0 = char(0-255),(-1 = not available)
        .globl  .IO_putc     #  $a0 = char(0-255)
        .globl  .IO_empty    #  ;$v0 = (0==empty)/(!0==!empty)
        .globl  .IO_flush    #  void
         
.IO_getc:  la   $v1,_IO_getc  ;  jr   $v1 ; nop
.IO_putc:  la   $v1,_IO_putc  ;  jr   $v1 ; nop
.IO_flush: la   $v1,_IO_flush ;  jr   $v1 ; nop 
.IO_empty: la   $v1,_IO_empty ;  jr   $v1 ; nop              
#-----------------Timer/Clock Stuff------------------------------------ 
        .globl   .TIMER_set       #  $a0 = compare_reg value     
        .globl   .TIMER_get       #  ;$v0 = compare_reg value, 
                                  #  ;$v1 = timer_reg value
        .globl   .TIMER_getClock  #  ;$v0 = clock value          
        .globl   .TIMER_display   #  $a0 = ON=1/OFF=0
                	
.TIMER_set:      la   $v1, _setTimer       ; jr   $v1 ; nop 
.TIMER_get:      la   $v1, _getTimer       ; jr   $v1 ; nop 
.TIMER_getClock: la   $v1, _getTmClock     ; jr   $v1 ; nop 
.TIMER_display:  la   $v1, _displayTimer   ; jr   $v1 ; nop             
#==================================================================
	  .globl __eoth
__eoth:
#=======END OF USER SPACE/PROGRAM LINKAGE/INTERFACE STUB===========
