# fileio.s
# 
# Demonstrates file manipulation system functions in PCSPIM
# 
# Prompts for a (full path) file name
# Creates a file with that name (will overwrite existing file)
# Writes the file's own name in the file
# Closes and then reopens the file for reading
# Reads from the file and echos the contents
# Closes the file and exits
#################################################################

	   .data			# Data declaration section
ibuffer: .space   80
obuffer: .space   80
prompt:  .asciiz  "Enter full file name: "
oerrmsg: .asciiz  "Error opening file"
werrmsg: .asciiz  "Error writing file"
rerrmsg: .asciiz  "Error reading file"
LF:      .asciiz  "\n"	

	   .text
main:					# Start of code section
	   li		$v0,4
	   la		$a0,prompt
	   syscall
	   li		$v0,8
	   la	      $a0,ibuffer
         li       $a1,80
	   syscall			# file name & LF now in ibuffer 
# remove LF
         la       $a0,ibuffer	# \ 
	   add      $a0,$a0,79	#  \ 
rLFloop: lb       $v0,0($a0)	#   \ Strip off
         bnez     $v0,rLFdone	#   / Line Feed
         sub      $a0,$a0,1	#  / 
         j        rLFloop	# /
rLFdone: sb       $0,0($a0)	# file name now in ibuffer
# file_open (create)
	   li		$v0,13	# file open function code
         la	      $a0,ibuffer # -> string with full path of file
	   li		$a1,0x4102  # flags: Text=0x4000, Create=0x100, R/W=0x2
	   li		$a2,0x180   # permissions: Read=0x100, Write=0x80
	   syscall
	   beq	$v0,-1,oerror # returns file descriptor or -1 if error
	   move     $s0,$v0	# good file descriptor in $s0
# file_write
	   li	      $v0,15	# file write function code
	   move     $a0,$s0	# file descriptor 
	   la		$a1,ibuffer # address of buffer
	   li		$a2,80	# amount to write
	   syscall
	   beq	$v0,-1,werror  # returns amount written or -1 if error (0=EOF)
# close / reopen for reading
         li       $v0,16	# file close function code
         move     $a0,$s0     # must do this since file pointer at EOF
         syscall
	   li		$v0,13	# now reopen so file pointer is at beginning
         la	      $a0,ibuffer # still -> string with full path of file
	   li		$a1,0x4000  # flags: Text=0x4000, Read=0x0
	   li		$a2,0x100   # permissions: Read=0x100
	   syscall
	   beq	$v0,-1,oerror  # returns file descriptor or -1 if error
	   move     $s0,$v0	# good file descriptor in $s0
# file_read
	   li		$v0,14	# file read function code
	   move	$a0,$s0	# file descriptor
	   la		$a1,obuffer # address of buffer
	   li	      $a2,80	# amount to read
	   syscall
	   beq	$v0,-1,rerror  # returns amount read or -1 if error (0=EOF)
# echo
	   li		$v0,4
	   la		$a0,LF
	   syscall
	   li		$v0,4
	   la		$a0,ibuffer
	   syscall
	   li		$v0,4
	   la		$a0,LF
	   syscall
	   li		$v0,4
	   la		$a0,obuffer
	   syscall
# file_close
         li       $v0,16	# file close function code
         move     $a0,$s0	# file descriptor
         syscall
exit:   
         li		$v0,10
	   syscall
oerror:  
         li		$v0,4
	   la		$a0,oerrmsg
	   syscall 
	   j		exit
werror:    
         li	      $v0,4
	   la		$a0,werrmsg
	   syscall
	   j		exit
rerror:  
         li		$v0,4
	   la	      $a0,rerrmsg
         syscall
	   j		exit

