# PolledIO.s
# 
# Demonstrates memory mapped I/O
#   Repeatedly prompts for a character with '>'
#   Echos printable characters until <Control-Z>
#   Prints '$' and exits (<Control-C> breaks)
# 
# Function getChar puts character into $v0
# Function putChar takes character in $a0
#
# Takes the place of syscall functions 11 & 12
# but function 12 echos the input and this doesn't
# 
	.text
main:                           # Start of code section
        li      $a0,0x0a
        jal     putChar
        li      $a0,'>'
        jal     putChar
        jal     getChar
        beq     $v0,0x1a,exit   # <CNTR-Z>
        move    $a0,$v0
        jal     putChar
        j       main
exit:
        li      $a0,'$'
        jal     putChar
        li      $v0,10          # exits program
        syscall
## gets character from keyboard into $v0
getChar:                        # <- entry point
        sub     $sp,$sp,4       # push changed register
        sw      $t0,0($sp)      # onto stack
        la      $t0,0xFFFF0000  # MEMORY MAPPED I/O
rrloop:
        lw      $v0,0($t0)      # read ready status (RSR)
        and     $v0,$v0,0x01    # mask the ready bit
        beqz    $v0,rrloop      # keep polling
# ready to read
        lw      $v0,4($t0)      # read the data byte (RDR)
        lw      $t0,0($sp)      # pop changed register
        add     $sp,$sp,4       # from stack
        jr      $ra             # return from getChar
## prints character in $a0 onto display
putChar:                        # <- entry point
        sub     $sp,$sp,8       # push changed
        sw      $t0,0($sp)      # registers
        sw      $t1,4($sp)      # onto stack
        la      $t0,0xFFFF0000  # MEMORY MAPPED I/O
beloop1:
        lw      $t1,8($t0)      # buffer empty status (TSR)
        and     $t1,$t1,0x01    # mask buffer empty bit	
        beqz    $t1,beloop1     # keep polling
# ready to write
        sw      $a0,12($t0)     # write to port (TDR)
beloop2:
        lw      $t1,8($t0)      # buffer emptied status (TSR)
        and     $t1,$t1,0x01    # mask buffer emptied bit
        beqz    $t1,beloop2     # waiting until empty again
# done writing
        lw      $t1,4($sp)      # pop changed
        lw      $t0,0($sp)      # registers
        add     $sp,$sp,8       # from stack
        jr      $ra             # return from putChar
 