# random.s gives us an array of pseudo-random numbers in the 
# range 0 <= n < 100 based on Knuth's advice. 
# X[n+1] <- (aX[n] + c) mod m
# using m = 2^16 = 65536 which works on a 16-bit machine well
# using a = 32445 so that a mod 8 = 5 and 99m/100 > a > m/100
# using c = 1
# using X0 = 31416 in $s0 as seed
	.text
main:
	li	$s0, 31416		# X0 - initialized (could be input)
	li	$t0, 32445		# a - initialized
      li    $t1, 0x0000FFFF   # mask to do modulo - initialized
	li	$t2, 396		# counter - initialized 4*100-4
	li	$t3, 100		# limit so values 0-99
Loop:	bltz	$t2, Exit		# Stop when counter passes zero
	mul	$s0, $s0, $t0     # X = aX
	addi	$s0, 1		# now X = aX+c
	and	$s0, $s0, $t1	# now X = (aX+c) mod m
	move	$a0, $s0		# save X in $s0  
	srl	$a0, $a0, 8		# divide by 256 (use upper 8 bits)
	bge	$a0, $t3, Loop	# only want those < 100
	li	$v0, 1		# op code for print_int
	syscall			# print value 0 <= X < 100
	sw    $a0, list+0($t2)  # store them in reverse order
	la	$a0, NL           # print a new line
	li	$v0, 4            # op code for print_str
	syscall
	add	$t2, $t2, -4	# decrease counter (4 bytes)
	j	Loop
Exit: li	$v0, 10           # standard return to OS
	syscall
# 
	.data
NL:	.asciiz  "\n"
      .word	   0xeeeeeeee	# also aligns to word boundary
list: .space   400            # room for 100 integers
	.word    0xffffffff     # also easy to find
