# randomfile.s
#
# Puts 100 pseudo-random numbers between 0 and 99 inclusive
# into file called "c:random.dat" 
# 
# X[n+1] <- (a*X[n] + c) mod m
# 
# using m = 2^16 = 65536
# using a = 32445 so that a mod 8 = 5 and 99m/100 > a > m/100
# using c = 1
# using X0 = 31416 in $s0
# 
# based on Knuth
# 
	.text
main:
	li	$s0, 31416	      # X0 - initalized (could be input)
	li	$t0, 32445	      # a - initialized
        li      $t1, 0x0000ffff       # mask to modulo - initialized
	li	$t2, 396	      # counter - initialized 4*100-4
	li	$t3, 100	      # limit so values 0-99
# Create file
	li	$v0,13	              # file open function code
        la	$a0,fname             # -> string with full path of file
	li	$a1,0x4101            # flags: Text=0x4000, Create=0x100, W=0x1
	li	$a2,0x80              # permissions: Write=0x80
	syscall
	beq	$v0,-1,oerror         # returns file descriptor or -1 if error
	move    $s1,$v0	              # good file descriptor in $s1
Loop:	bltz	$t2, Store	      # End loop when counter passes zero
	mul	$s0, $s0, $t0	      # X = aX
	addi	$s0, 1		      # now X = aX+c
	and	$s0, $s0, $t1	      # now X = (aX+c) mod m
	move	$a0, $s0	      # keep X in $s0  
	srl	$a0, $a0, 8	      # divide by 256 (use upper 8 bits)
	bge	$a0, $t3, Loop	      # only want those < 100
	sw	$a0,buffer+0($t2)     # save it (in reverse order)
	add	$t2, $t2, -4	      # decrease counter (4 bytes)
	j	Loop 
# file_write
Store:  li	$v0,15	              # file write function code
	move    $a0,$s1	              # file descriptor 
	la	$a1,buffer            # address of buffer
	li	$a2,400	              # amount to write
	syscall
	beq	$v0,-1,werror         # returns amount written or -1 if error (0=EOF)
# 
# file_close
Exit:   li      $v0,16	              # file close function code
        move    $a0,$s1	              # file descriptor
        syscall
        li	$v0, 10               # standard return to OS
	syscall
# 
oerror: la	$a0,oerrmsg
	li	$v0,4
	syscall
	j	Exit
werror: la	$a0,werrmsg
	li	$v0,4
	syscall
        j       Exit
#
	.data
fname:  .asciiz  "c:random.dat"
oerrmsg: .asciiz "Error opening file\n"
werrmsg: .asciiz "Error writing file\n"
        .word	 0xeeeeeeee	      # aligns to word boundary
buffer: .space   400    
	.word    0xffffffff           # also easy to find in PCSPIM
