# URAND.s
# 
# Uniformly distributed Random Number Generator of L'Ecuyer
# for 32-bit computers. See "Efficient and Portable 
# Combined Random Number Generators", CACM 31 (June 1988). 
# 
	  .data                   # Data declaration section
title:  .asciiz   "1000 Random Numbers\n"
const:  .float    4.656613e-10
LF:     .asciiz   "\n"
# 
	  .text
main:                           # Start of code section
        li        $a1, 31415926 # Initialize
        li        $a2, 27182818 # Seeds
# 
        li        $v0, 4     
        la        $a0, title
        syscall                 # Print title
# 
        li        $s0, 1000     # How many numbers to print
loop:   beqz      $s0, exit
        jal       urand         # Get next random
        li        $v0, 2        # print_float code
        mov.s     $f12,$f0      # It was in $f0
        syscall                 # Print it
        la        $a0, LF
        li        $v0, 4        # Print Line Feed
        syscall
        sub       $s0, $s0, 1   # Decrement counter
        j         loop
# 
exit:   li        $v0, 10
        syscall
###
urand:                          # <- Entry point
# Takes two seed values in $a1 and $a2.
# Modifies both of them and returns the
# next random number in $f0 as expected.
# Does not save $t0, $t1, $t2, or $f12.
        div       $t0, $a1, 53668
        mul       $t1, $t0, 53668
        mul       $t2, $t0, 12211
        sub       $a1, $a1, $t1
        mul       $a1, $a1, 40014
        sub       $a1, $a1, $t2
        bgez      $a1, L1
        add       $a1, $a1, 2147483563
L1:     div       $t0, $a2, 52774
        mul       $t1, $t0, 52774
        mul       $t2, $t0, 3791
        sub       $a2, $a2, $t1
        mul       $a2, $a2, 40692
        sub       $a2, $a2, $t2
        bgez      $a2, L2
        add       $a2, $a2, 2147483399
L2:     sub       $t0, $a1, $a2
        blez      $t0, L3
        b         convert
L3:     add       $t0, $t0, 2147483562
# 
convert: 
        mtc1      $t0, $f0
        cvt.s.w   $f0, $f0
        l.s       $f12, const
        mul.s     $f0, $f12, $f0
# 
        jr        $ra


